/*	   =================================================
	   SUP3D.C - Construao do grfico de uma superfcie
	   =================================================               

        (C) 1996, Lenimar Nunes de Andrade, lenimar@mat.ufpb.br

*/

#include <graphics.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>

#define  real       float
#define  integer    int
#define  vazio      void
#define  procedure  void

#define  ESC            27
#define  SetaPCima      72
#define  SetaPBaixo     80
#define  SetaPEsquerda  75
#define  SetaPDireita   77
#define  TeclaF1        59
#define  TeclaF2        60
#define  TeclaF3        61
#define  TeclaF4        62
#define  TeclaF5        63


/* VARIAVEIS GLOBAIS */
real A, B,                   /* Coordenadas do centro da tela                */
     TAM = 30,               /* Coeficiente de ampliaao                     */
     nu = 12,                /* Quant. subdivisoes no domnio da varivel u  */
     nv = 12,                /* Quant. subdivisoes no domnio da varivel v  */
     fi = -M_PI/7,           /* Angulo de rotaao em torno do eixo y         */
     teta = -M_PI/7,         /* Angulo de rotaao em torno do eixo z         */
     proj_X, proj_Y;         /* Coordenadas da projeao do ponto (x, y, z)   */

/* ------------------------------------------------------------------------- */

/* Definiao da parametrizaao   F(u,v) = (f1(u,v), f2(u,v), f3(u,v))
   da superfcie cujo grfico e' desenhado. Neste caso, temos  a parametri-
   zaao do paraboloide hiperbolico  F(u, v) = (u, v, (-u*u + v*v)/4))       */

real f1(real u, real v) { return u; }
real f2(real u, real v) { return v; }
real f3(real u, real v) { return (-u*u + v*v)/4; }

/* ------------------------------------------------------------------------- */

procedure IniciaModoGrafico(vazio) {

/* Inicializa o modo grfico. O arquivo .BGI associado ao tipo de video
   utilizado (Ex.: EGAVGA.BGI) precisa estar no subdiretorio atual. */

  integer gm, gd = DETECT;

  initgraph(&gd, &gm, "");
  if (graphresult() != grOk) exit(1);   /* Encerra a execuao do programa
			    se houver erro na mudanca para o modo grfico */
}

/* ------------------------------------------------------------------------ */

procedure ProjetaPonto(real x, real y, real z) {

/* Calcula as coordenadas do ponto (x, y, z) no plano de projeao. E' feita
   uma ampliaao de TAM unidades e uma translaao da origem  do  sistema de
   coordenadas do plano de projeao para o ponto (A, B). */

  real X, Y;

  /* Gira (x, y, z) de teta radianos em torno do eixo z e de fi radianos
     em torno do eixo y. E' feita uma projeao ortografica na direao x. */

  X = y*cos(teta) + x*sin(teta);
  Y = z*cos(fi) + x*cos(teta)*sin(fi) - y*sin(fi)*sin(teta);

  /* Ampliaao e translaao de (X, Y) */
  proj_X = A + X*TAM;
  proj_Y = B - Y*TAM;
}

/* ------------------------------------------------------------------------- */

procedure DesenhaEixos(integer k, integer cor1, integer cor2) {

/* Desenha os eixos x, y, z usando tamanho k e cores especificadas */

  real X0, Y0;

  ProjetaPonto(0, 0, 0);         /* Calcula a projeao da origem (0, 0, 0) */
  X0 = proj_X, Y0 = proj_Y;

  setcolor(cor1);
  ProjetaPonto(k, 0, 0);
  line(X0, Y0, proj_X, proj_Y);  /* Desenha o eixo x */
  setcolor(cor2);
  outtextxy(proj_X, proj_Y, "x");

  setcolor(cor1);
  ProjetaPonto(0, k, 0);
  line(X0, Y0, proj_X, proj_Y);  /* Desenha o eixo y */
  setcolor(cor2);
  outtextxy(proj_X, proj_Y, "y");

  setcolor(cor1);
  ProjetaPonto(0, 0, k);
  line(X0, Y0, proj_X, proj_Y);  /* Desenha o eixo z */
  setcolor(cor2);
  outtextxy(proj_X, proj_Y, "z");
}

/* ------------------------------------------------------------------------- */

procedure DesenhaSuperficie(real umin, real umax, real vmin, real vmax,
							     integer cor) {

/* Usando a  cor especificada, desenha a superfcie definida pelas  equaoes
   parametricas (f1(u, v), f2(u, v), f3(u, v)) usando pequenos quadrilateros.
   E' usado como domnio o retangulo [umin, umax] x [vmin, vmax]. */

  real u, v, x, y, z, incrU, incrV;
  integer X1, X2, X3, X4, Y1, Y2, Y3, Y4;

  incrU = (umax - umin)/nu;   /* Incremento da varivel u */
  incrV = (vmax - vmin)/nv;   /* Incremento da varivel v */

  setcolor(cor);

  u = umin + incrU;

  while (u < umax + incrU/2) {       /* Se nao houvesse erros de aproximaao
		esse "while" poderia ser substituido por "while (u <= umax)" */

    v = vmin + incrV;

    while (v < vmax + incrV/2) {     /* Se nao houvesse erros de aproximaao
		esse "while" poderia ser substituido por "while (v <= vmax)" */

      x = f1(u, v);
      y = f2(u, v);
      z = f3(u, v);
      ProjetaPonto(x, y, z);
      X1 = proj_X; Y1 = proj_Y;

      x = f1(u - incrU, v);
      y = f2(u - incrU, v);
      z = f3(u - incrU, v);
      ProjetaPonto(x, y, z);
      X2 = proj_X; Y2 = proj_Y;

      x = f1(u, v - incrV);
      y = f2(u, v - incrV);
      z = f3(u, v - incrV);
      ProjetaPonto(x, y, z);
      X3 = proj_X; Y3 = proj_Y;

      x = f1(u - incrU, v - incrV);
      y = f2(u - incrU, v - incrV);
      z = f3(u - incrU, v - incrV);
      ProjetaPonto(x, y, z);
      X4 = proj_X; Y4 = proj_Y;

      line (X1, Y1, X2, Y2);
      line (X4, Y4, X2, Y2);
      line (X3, Y3, X4, Y4);
      line (X3, Y3, X1, Y1);

      v = v + incrV;
    }          /* fim do "while (v < ... " */

    u = u + incrU;
  }            /* fim do "while (u < ... " */
}

/* ------------------------------------------------------------------------- */

procedure main(vazio) {                     /* Funao principal              */

  float angulo = M_PI/20.0;
  int ch, cor = RED;

  printf("\n# Use as teclas de seta para girar a superficie em tornos dos eixos\n");
  printf("\n# Pressione ESC para encerrar a execucao do programa\n");
  printf("\n# Pressione qualquer tecla para continuar\n");
  getch();

  IniciaModoGrafico();
  A = getmaxx()/2, B = getmaxy()/2;         /* Define  (A, B)  como sendo as
					       coordenadas do centro da tela */

  do {
    DesenhaSuperficie(-3, 3, -3, 3, RED);
    DesenhaEixos(3, YELLOW, WHITE);

    ch = getch();

    if (ch == 0) {
      ch = getch();
      DesenhaSuperficie(-3, 3, -3, 3, 0);
      DesenhaEixos(3, 0, 0);

      switch (ch) {
	case SetaPDireita:
	  teta -= angulo; break;
	case SetaPEsquerda:
	  teta += angulo; break;
	case SetaPCima:
	  fi -= angulo; break;
	case SetaPBaixo:
	  fi += angulo; break;
	case TeclaF1:
	  cor = 1 + (cor + 1) % 15; break;
	case TeclaF2:
	  TAM *= 1.25; break;
	case TeclaF3:
	  TAM /= 1.25; break;
	case TeclaF4:
	  teta *= 1.25; break;
	case TeclaF5:
	  fi *= 1.25; break;
      }

    }
  } while (ch != ESC);

  closegraph();
}

/* ------------------------------------------------------------------------- */

/*** FIM DE "SUP_ANM.C" ***/

